package SVK;

our $VERSION = '1.05';

=head1 NAME

SVK::Version - The SVK project-wide version number

=head1 SYNOPSIS

    package SVK::Whatever;

    # Must be on one line so MakeMaker can parse it.
use SVK::Version;  our  = ;

=head1 DESCRIPTION

Because of the problems coordinationg revision numbers in a distributed
version control system and across a directory full of Perl modules, this
module provides a central location for the project's release number.

=cut

1;
